function cqtKernel = genCQTkernel(fmax, bins, fs, varargin)
%Calculating the CQT Kernel for one octave. All atoms are center-stacked.
%Atoms are placed so that the stacks of lower octaves are centered at the
%same positions in time, however, their amount is reduced by factor two for
%each octave down.
%
%INPUT:
%   fmax          ... highest frequency of interest
%   bins          ... number of bins per octave
%   fs            ... sampling frequency
%
%optional input parameters (parameter name/value pairs):
%
%   'q'             ... Q scaling factor. Default: 1.
%   'atomHopFactor' ... relative hop size corresponding to the shortest
%                       temporal atom. Default: 0.25.
%   'thresh'        ... values smaller than 'tresh' in the spectral kernel are rounded to
%                       zero. Default: 0.0005.
%   'oversampTwo'   ... if set to 1, time-domain oversampling of factor 2 is applied. 
%                       That is, twice as many sampling points are computed (in order to
%                       allow for interoctave phase adjustments)
%   'allowSevAtoms' ... if set to 1, the generated kernel may contain
%                       more than one atom per FFT frame thus increasing
%                       the performance of the CQT. If time-scaling using
%                       the CQT phase vocoder method is about to be
%                       applied, allowSevAtoms has to be set to 0 as the
%                       hop size will be altered during resynthesis.
%                       Default: 1.
%   'perfRast'      ... if set to 1 the kernel is designed in order to
%                       enable perfect rasterization using the function
%                       cqtPerfectRast() (Default: perfRast=0). See documentation of
%                       'cqtPerfectRast' for further information.
%
%OUTPUT:
%   cqtKernel   ... Structure that contains the spectral kernel 'fKernel'
%                   additional design parameters used in cqt(), cqtPerfectRast() and icqt().
%
%Christian Schrkhuber, Anssi Klapuri 2010-06
%2011-08: 'allowSevAtoms' parameter added, 
%         oversampling of factor 2 implemented ('oversampTwo'),
%         window function option removed: the used window function now is
%         modhann(N)

%% input parameters
q = 1; %default value
atomHopFactor = 0.25; %default value
thresh = 0.0005; %default value
perfRast = 0; %default value
allowSevAtoms = 1;
oversampTwo = 0;

for ain = 1:length(varargin)
    if strcmp(varargin{ain},'q'), q = varargin{ain+1}; end;
    if strcmp(varargin{ain},'atomHopFactor'), atomHopFactor = varargin{ain+1}; end;
    if strcmp(varargin{ain},'thresh'), thresh = varargin{ain+1}; end;
    if strcmp(varargin{ain},'allowSevAtoms'), allowSevAtoms = varargin{ain+1}; end;
    if strcmp(varargin{ain},'perfRast'), perfRast = varargin{ain+1}; end;
    if strcmp(varargin{ain},'oversampTwo'), oversampTwo = varargin{ain+1}; end;
end

%% define
fmin = (fmax/2)*2^(1/bins);
Q = 1/(2^(1/bins)-1);
Q = Q*q;
Nk_max = Q * fs / fmin; 
Nk_max = 2 * round((Nk_max+1)/2) - 1; %modhann windows are always of odd length


%% Compute FFT size, FFT hop, atom hop, ...
Nk_min = Q * fs / (fmin*2^((bins-1)/bins)); %length of the shortest atom [samples]
atomHOP = 2*round(Nk_min*atomHopFactor/2); %force atomHOP to be even so that for the shifted kernel (oversampling 2->atomHop/2) no fractional delay occurs

first_center = ceil(Nk_max/2); %first possible center position within the frame
first_center = atomHOP * ceil(first_center/atomHOP); %lock the first center to an integer multiple of the atom hop size
FFTLen = 2^nextpow2(first_center+ceil(Nk_max/2)); %use smallest possible FFT size (increase sparsity)

if perfRast
    winNr = floor((FFTLen-ceil(Nk_max/2)-first_center)/atomHOP); %number of temporal atoms per FFT Frame
    if winNr == 0
        FFTLen = FFTLen * 2;
        winNr = floor((FFTLen-ceil(Nk_max/2)-first_center)/atomHOP);
    end
elseif oversampTwo
    winNr = floor((FFTLen-first_center+1-atomHOP/2-(Nk_max-1)/2)/atomHOP)+1; %number of temporal atoms per FFT Frame
    if winNr == 0
        FFTLen = FFTLen * 2;
        winNr = floor((FFTLen-first_center+1-atomHOP/2-(Nk_max-1)/2)/atomHOP)+1;
    end    
else
    winNr = floor((FFTLen-ceil(Nk_max/2)-first_center)/atomHOP)+1; %number of temporal atoms per FFT Frame
end

if allowSevAtoms == 0
    winNr = 1; 
end

last_center = first_center + (winNr-1)*atomHOP;
fftHOP = (last_center + atomHOP) - first_center; % hop size of FFT frames
fftOLP = (FFTLen-fftHOP/FFTLen)*100; %overlap of FFT frames in percent ***AK:needed?

%% init variables
tempKernel= zeros(1,FFTLen); 
sparKernel= []; 

%% Compute kernel
atomInd = 0;
for k = 1:bins
   fk = fmin*2^((k-1)/bins); 
   Nk = Q*fs/fk;
   winFct = modhann(Nk);
   n = (0:length(winFct)-1) - (length(winFct)-1)/2; 
   tempKernelBin = (winFct/sum(winFct)).* exp(-1i*2*pi*n*fk/fs);
   atomOffset = first_center-ceil(Nk/2);

   for i = 1:winNr 
       shift = atomOffset + ((i-1) * atomHOP);
       tempKernel(1+shift:length(tempKernelBin)+shift) = tempKernelBin; 
       atomInd = atomInd+1;
       specKernel= fft(conj(tempKernel));      
       specKernel(abs(specKernel)<=thresh)= 0;   
       sparKernel= sparse([sparKernel; specKernel]);
       tempKernel = zeros(1,FFTLen); %reset window     
       
   end
end
sparKernel = (sparKernel.')/FFTLen;

%% Normalize the magnitudes of the atoms
[ignore,wx1]=max(sparKernel(:,1));
[ignore,wx2]=max(sparKernel(:,end));
wK=sparKernel(wx1:wx2,:);
wK = diag(wK * wK');
wK = wK(round(1/q)+1:(end-round(1/q)-2));
weight = 1./mean(abs(wK));
weight = weight.*(fftHOP/FFTLen); 
weight = sqrt(weight); %sqrt because the same weight is applied in icqt again
sparKernel = weight.*sparKernel;

%% return
cqtKernel = struct('fKernel',sparKernel,'fftLEN',FFTLen,'fftHOP',fftHOP,'fftOverlap',fftOLP,'perfRast',perfRast,...
    'bins',bins,'firstcenter',first_center,'atomHOP',atomHOP,'atomNr',winNr,'Nk_max',Nk_max,'Q',Q,'fmin',fmin);
